@@ -26,14 +26,15 @@ import time
 import datetime
 
 from optparse import OptionParser
+from xdg import BaseDirectory
 
 # Configuration and sound data paths
-import paths
+from . import paths
 
 def report(msg):
     """Output information messages for the user on stdout
     and if desired, by espeak synthesis"""
-    print msg
+    print(msg)
     if options.use_espeak_synthesis:
         os.system("espeak \"" + msg + "\"")
 
@@ -57,7 +58,7 @@ def question(text, default):
         input_audio_icon()
 
         if not options.dont_ask:
-            str_inp = raw_input(">") 
+            str_inp = input(">")
 
         # On plain enter, return default
         if options.dont_ask or (len(str_inp) == 0):
@@ -75,6 +76,8 @@ def question(text, default):
             return int(str_inp)
         elif isinstance(default, str):
             return str_inp
+        else:
+            raise TypeError("Invalid type for the default value")
 
 def question_with_suggested_answers(text, default, suggest):
     """Ask a question with suggested answers. If the answer typed is not
@@ -200,7 +203,7 @@ class Options(object):
             },
         'dont_ask':
             {
-            'descr' : "Do not ask any questions, allways use default values",
+            'descr' : "Do not ask any questions, always use default values",
             'doc' : None,
             'type' : bool,
             'default' : False,
@@ -222,17 +225,17 @@ A simple dialog based tool for basic configuration of Speech Dispatcher
 and problem diagnostics."""
         self.cmdline_parser = OptionParser(usage)
 
-        for option, definition in self._conf_options.iteritems():
+        for option, definition in self._conf_options.items():
             # Set object attributes to default values
             def_val = definition.get('default', None)
             setattr(self, option, def_val)
             
             # Fill in the cmdline_parser object
-            if definition.has_key('command_line'):
+            if 'command_line' in definition:
                 descr = definition.get('descr', None)                
                 type = definition.get('type', None)
                 
-                if definition.has_key('arg_map'):
+                if 'arg_map' in definition:
                     type, map = definition['arg_map']
                 if type == str:
                     type_str = 'string'
@@ -257,10 +260,10 @@ and problem diagnostics."""
         # Set options according to command line flags
         (cmdline_options, args) = self.cmdline_parser.parse_args()
 
-        for option, definition in self._conf_options.iteritems():
+        for option, definition in self._conf_options.items():
                 val = getattr(cmdline_options, option, None)
                 if val != None:
-                    if definition.has_key('arg_map'):
+                    if 'arg_map' in definition:
                         former_type, map = definition['arg_map']
                         try:
                             val = map[val]
@@ -279,17 +282,9 @@ class Tests:
     def __init__(self):
         self.festival_socket = None
 
-    def user_speechd_dir(self):
-        """Return user Speech Dispatcher configuration and logging directory"""
-        return os.path.expanduser(os.path.join('~', '.speech-dispatcher'))
-
-    def user_speechd_dir_exists(self):
-        """Determine whether user speechd directory exists"""
-        return os.path.exists(self.user_speechd_dir())
-
     def user_conf_dir(self):
         """Return user configuration directory"""
-        return os.path.join(self.user_speechd_dir(), "conf")
+        return os.path.join(BaseDirectory.xdg_config_home, "speech-dispatcher")
 
     def system_conf_dir(self):
         """Determine system configuration directory"""
@@ -307,7 +302,8 @@ class Tests:
         self.festival_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         try:
             self.festival_socket.connect((socket.gethostbyname(host), port))
-        except socket.error, (num, reson):
+        except socket.error as e:
+            (num, reson) = e.args
             report("""ERROR: It was not possible to connect to Festival on the
 given host and port. Connection failed with error %d : %s .""" % (num, reson))
             report("""Hint: Most likely, your Festival server is not running now
@@ -323,8 +319,8 @@ Try /etc/init.d/festival start or run 'festival --server' from the command line.
         if not self.festival_socket:
             if not self.festival_connect():
                 return False
-        self.festival_socket.send("(require 'speech-dispatcher)\n")
-        reply = self.festival_socket.recv(1024)
+        self.festival_socket.send(bytes("(require 'speech-dispatcher)\n", "ascii"))
+        reply = str(self.festival_socket.recv(1024))
         if "LP" in reply:
             report("Festival contains freebsoft-utils.")
             return True
@@ -355,8 +351,8 @@ Search for package like python-speechd, download and install it""")
             cmd = "aplay" + " " + wavfile
         elif type == 'pulse':
             cmd = "paplay" + " " + wavfile
-        elif type == 'oss':
-            raise NotImplementedError
+        else:
+            raise NotImplementedError("Test for this audio system is not implemented")
 
         try:
             ret = os.system(cmd)
@@ -444,7 +440,7 @@ is not installed.""")
         report("Testing PULSE sound output")
         return self.audio_try_play(type='pulse')
 
-    def test_alsa(self):
+    def test_libao(self):
         """Test libao sound output"""
         report("Testing libao sound output")
         return self.audio_try_play(type='libao')
@@ -501,7 +497,7 @@ what is wrong)""", True):
                 working_audio += ["pulse"]
 
         if decide_to_test('libao', "libao sound system", audio_output): 
-            if self.test_pulse():
+            if self.test_libao():
                 working_audio += ["libao"]
 
         report("Testing whether Python Speech Dispatcher library is in path and importable")
@@ -518,18 +514,18 @@ what is wrong)""", True):
         report("""
 
 Diagnostics results:""")
-        if results.has_key('spd_say_working'):
+        if 'spd_say_working' in results:
             if results['spd_say_working']:
                 report("Speech Dispatcher is working")
             else:
                 report("Speech Dispatcher not working through spd-say")
-        if results.has_key('synthesizers'):
+        if 'synthesizers' in results:
             report("Synthesizers that were tested and seem to work: %s" %
                    str(results['synthesizers']))
-        if results.has_key('audio'):
+        if 'audio' in results:
             report("Audio systems that were tested and seem to work: %s" %
                    str(results['audio']))
-        if results.has_key('python_speechd'):
+        if 'python_speechd' in results:
             if(results['python_speechd']):
                 report("Python Speech Dispatcher module is importable")
             else:
@@ -569,6 +565,8 @@ Do you want to debug 'system' or  'user' Speech Dispatcher?""",
 processes. Do you want to do it now?""", True)
         if reply:
             os.system("killall speech-dispatcher")
+            # Attempt to workaround the psmisc 22.15 bug with 16 char max process names
+            os.system("killall speech-dispatch")
         else:
             report("""
 You decided not to kill running Speech Dispatcher processes.
@@ -580,7 +578,7 @@ Please make sure your Speech Dispatcher is not running now.""")
         time.sleep(2)
 
         # All debugging files are written to TMPDIR/speech-dispatcher/
-        if os.environ.has_key('TMPDIR'):
+        if 'TMPDIR' in os.environ:
             tmpdir = os.environ['TMPDIR']
         else:
             tmpdir = "/tmp/"
@@ -607,7 +605,7 @@ Please start your system Speech Dispatcher now with parameter '-D'""")
         time.sleep(2)
 
         if not speechd_started:
-            reply = report("Speech Dispatcher failed to start, continuing anyway")
+            reply = question("Speech Dispatcher failed to start, continuing anyway")
 
         report("Trying to speak some messages")
         ret = os.system("spd-say \"Speech Dispatcher debugging 1\"")
@@ -626,6 +624,8 @@ Please start your system Speech Dispatcher now with parameter '-D'""")
         os.system("tar -cz %s %s > %s" % 
                   (debugdir_path, configure_directory, debugarchive_path))   
         os.system("killall speech-dispatcher")
+        # Attempt to workaround the psmisc 22.15 bug with 16 char max process names
+        os.system("killall speech-dispatch")
         os.system("rm -rf %s" % debugdir_path)
 
         report("""
@@ -657,7 +657,7 @@ class Configure:
         # Parse config file in-place and replace the desired options+values
         for line in fileinput.input(configfile, inplace=True, backup=".bak"):
             # Check if the current line contains any of the desired options
-            for opt, value in options.iteritems():
+            for opt, value in options.items():
                 if opt in line:
                     # Now count unknown words and try to judge if this is
                     # real configuration or just a comment
@@ -689,42 +689,35 @@ class Configure:
                         else:
                             spd_val = str(value)
                             
-                        print opt + "   " + spd_val
+                        print(opt + "   " + spd_val)
                         break
             
             else:
-                print line,
+                print(line, end=' ')
                 
     def create_user_configuration(self):
         """Create user configuration in the standard location"""
 
         # Ask before touching things that we do not have to!
-        if test.user_speechd_dir_exists():
-            if test.user_conf_dir_exists():
-                if test.user_configuration_seems_complete():
-                    reply = question(
-                        """User configuration already exists.
-Do you want to rewrite it with a new one?""", False)
-                    if reply == False:
-                        report("Keeping configuration intact and continuing with settings.")
-                        return
-                    else:
-                        self.remove_user_configuration()
+        if test.user_conf_dir_exists():
+            if test.user_configuration_seems_complete():
+                reply = question(
+                    "User configuration already exists."
+                    "Do you want to rewrite it with a new one?", False)
+                if reply == False:
+                    report("Keeping configuration intact and continuing with settings.")
+                    return
                 else:
-                    reply = question(
-                        """User configuration already exists, but it seems to be incomplete.
-Do you want to keep it?""", False)
-                    if reply == False:
-                        self.remove_user_configuration()
-                    else:
-                        report("Keeping configuration intact and aborting.")
-                        return
-
-            # TODO: Check for permissions on logfiles and pid
-        else:
-            report("Creating " + test.user_speechd_dir())
-            os.mkdir(test.user_speechd_dir())
-
+                    self.remove_user_configuration()
+            else:
+                reply = question(
+                    "User configuration already exists, but it seems to be incomplete."
+                    "Do you want to keep it?", False)
+                if reply == False:
+                    self.remove_user_configuration()
+                else:
+                    report("Keeping configuration intact and aborting.")
+                    return
         # Copy the original intact configuration files
         # creating a conf/ subdirectory
         shutil.copytree(paths.SPD_CONF_ORIG_PATH, test.user_conf_dir())
@@ -740,7 +733,7 @@ Do you want to keep it?""", False)
             report("Warning: You must be root or under sudo to do this.")
             report("Configuring system settings for Speech Dispatcher")
         else:
-            assert(0)
+            raise ValueError("Invalid configuration type")
 
         # Now determine the most important config option
         self.default_output_module = question_with_suggested_answers(
@@ -755,7 +748,7 @@ Do you want to keep it?""", False)
         self.default_audio_method = question_with_suggested_answers(
             "Default audio output method",
             "libao",
-            ["libao", "pulse", "alsa", "oss", "pulse,alsa"])
+            ["pulse", "libao", "alsa", "oss", "pulse,alsa"])
 
         self.default_speech_rate = question(
             "Default speech rate (on the scale of -100..100, 0 is default, 50 is faster, -50 is slower)",
@@ -806,6 +799,8 @@ use special audio settings, non-standard synthesizer ports etc.""" % configfile)
 Do you want to kill all running Speech Dispatchers and try again?""", True)
             if reply:
                 os.system("killall speech-dispatcher")
+                # Attempt to workaround the psmisc 22.15 bug with 16 char max process names
+                os.system("killall speech-dispatch")
                 err = os.system("speech-dispatcher")
                 if err:
                     report("Can't start Speech Dispatcher")
@@ -852,7 +847,7 @@ you have to start it manually to continue.""")
         elif speechd_type == 'system':
             self.configure_basic_settings(type='system')
         else:
-            assert(False)
+            raise ValueError("Invalid configuration type")
 
         reply = question("Do you want to start/restart Speech Dispatcher now and run some tests?", True)
         if not reply:
